/* Example Simple DFT Analyser*/
/**/

/* 
Inputs: in_signal
Outputs: dft_mag
Parameters: OutputOrder
*/

#include "IEC_DLL.h"
#include "ext_simenv_capi.h"
#include <stdlib.h>
#include <math.h>
#include <stdio.h> 
#include <string.h> 


#define INT_MEM ((internal_memory *)pInstanceCapi->MiscStates)

/* internal_memory */
typedef struct {
	real64_T OutputOrder,dT;
	char_T last_error_message[255];
	double debug;
	double data1[256]; // hardcoded buffer size - make sure it is the same as BUF_SIZ
	int counter;
	double OutputOrderMagnitude;
} internal_memory;

/* static_inputs */
const StaticESEInputSignal static_inputs[] = {
	{ "in_signal", "", 1 }
};

/* static_outputs */
const StaticESEOutputSignal static_outputs[] = {
	{ "dft_mag", "", 1 },
	{ "debug", "", 1 }
};

/* static_parameters */
const StaticESEParameter static_parameters[] = {
	{ "OutputOrder", "Harmonic order being calculated (1,2,..)", "n/a", 1, 0., 1000. }
};


/* static_model_info */
StaticExtSimEnvCapi static_model_info = {
	{0, 0, 0, 1},	// const uint8_T APIRelease[4]: Release number of the API used during code generation
	"Example IEC DLL for DFT Computation (50Hz variant)",	// const char_T * const ModelName: Model name
	"R01-V01",			// const char_T * const ModelVersion: Model version
	"Example IEC DLL for DFT Computation",	// const char_T * const ModelDescription: Model description
	"",				// const char_T * const VersionControlInfo: Version control information
	"-PFPrecise",	// const char_T * const GeneralInformation: General info 
	"2021-05-12",	// const char_T * const ModelCreated: Model created on
	"A. Constantin",	// const char_T * const ModelCreator: Model created by
	"2021-05-12",	// const char_T * const ModelLastModifiedDate:  Model last modified on
	"Last Editor",	// const char_T * const ModelLastModifiedBy: Model last modified by
	"Visual Express 2013",	// const char_T * const ModelModifiedComment: Model modified comment
	"",				// const char_T * const ModelModifiedHistory: Model modified history
	"2021-05-12",				// const char_T * const CodeGeneratedOn: Code generated on
	"",				// const char_T * const IncludedSolver: Solver name (can be empty)
	1./(50. * 256.),			// const real64_T FixedStepBaseSampleTime: Base sample time // hardcoded sample time size
	sizeof(static_inputs)/sizeof(static_inputs[0]),	// const int32_T NumInputPorts: Number of inputs
	&static_inputs[0],	// const StaticESEInputSignal * const InputPortsInfo: Pointer to input signal description array
	sizeof(static_outputs)/sizeof(static_inputs[0]),	// const int32_T NumOutputPorts: Number of outputs
	&static_outputs[0],	// const StaticESEOutputSignal * const OutputPortsInfo: Pointer to output signal description array
	sizeof(static_parameters)/sizeof(static_parameters[0]),	// const int32_T NumParameters: Number of parameters
	&static_parameters[0],	// const StaticESEParameter * const ParametersInfo: Pointer to parameter description array
	0,				// const int32_T NumContStates: Number of continuous states
	sizeof(internal_memory),	// const int32_T SizeofMiscStates: Size of work variables / misc states
	{0, 0, 0, 0},	// const uint32_T ModelChecksum[4]:  Model checksum (user defined)
	NULL,			// const char_T *LastErrorMessage: Error string pointer
	3,				// const uint8_T EMT_RMS_Mode: Mode: EMT = 1, RMS = 2, EMT & RMS = 3, otherwise: 0
	0,				// const uint8_T LoadflowFlag: Model contains a loadflow function: 0 = no, 1 = yes
	0				// ESEExtension Extension: Provided for extensions
};





/* Model_GetInfo */
extern DLL_EXPORT const StaticExtSimEnvCapi* __cdecl Model_GetInfo(void)
{
	return &static_model_info;
}


/* Model_Instance */
extern DLL_EXPORT InstanceExtSimEnvCapi* __cdecl Model_Instance(uint32_T UseSolverInDLL, real64_T Ta)
{
	uint8_T i, w;

	InstanceExtSimEnvCapi * pInstanceCapi;

	pInstanceCapi = (InstanceExtSimEnvCapi *)malloc(sizeof(InstanceExtSimEnvCapi));
	if (pInstanceCapi == NULL) {
		return NULL;
	}

	/* allocate internal memory */
	/* ExternalInputs, taking into account width of vectorized signals */
	w = 0;
	for (i = 0; i < static_model_info.NumInputPorts; i++) {
		w = w + (&static_model_info)->InputPortsInfo[i].Width;
	}
	pInstanceCapi->ExternalInputs = (real64_T *)malloc(w*sizeof(real64_T));
		
	/* ExternalOutputs, taking into account width of vectorized signals */
	w = 0;
	for (i = 0; i < static_model_info.NumOutputPorts; i++) {
		w = w + (&static_model_info)->OutputPortsInfo[i].Width;
	}
	pInstanceCapi->ExternalOutputs = (real64_T *)malloc(w*sizeof(real64_T));
	
	/* Parameters, only scalar entries */
	pInstanceCapi->Parameters = (real64_T *)malloc(
		static_model_info.NumParameters*sizeof(real64_T));
	
	/* ContinuousStates, not used in current version, set NULL */
	pInstanceCapi->ContinuousStates = NULL;

	/* StateDerivatives, not used in current version, set NULL */
	pInstanceCapi->StateDerivatives = NULL;

	/* MiscStates, pointer to internal_memory */
	pInstanceCapi->MiscStates = (uint8_T *)malloc(static_model_info.SizeofMiscStates);

	/* LastErrorMessage, initialize to blank */
	pInstanceCapi->LastErrorMessage = NULL;
	memset(&(INT_MEM->last_error_message), 0, sizeof(INT_MEM->last_error_message));

	/* @TODO: implement mechanism for LastGeneralMessage */
	pInstanceCapi->LastGeneralMessage = NULL;

	/* VerboseLevel, presently not used */
	pInstanceCapi->VerboseLevel = 0;

	/* Extension, not used in current version */
	pInstanceCapi->Extension.UserVoidPtr = NULL;

	/* set the integration time-step size */
	if (Ta > 0) {
		INT_MEM->dT = Ta;
	} else {
		INT_MEM->dT = static_model_info.FixedStepBaseSampleTime;
	}	

	return pInstanceCapi;
}


/* Model_Check_Parameters */
extern DLL_EXPORT const char* __cdecl Model_CheckParameters(InstanceExtSimEnvCapi *pInstanceCapi)
{
	uint8_T i;

	for (i = 0; i < static_model_info.NumParameters; i++) {
		if (pInstanceCapi->Parameters[i] < (&static_model_info)->ParametersInfo[i].MinValue) {
			_snprintf(INT_MEM->last_error_message, sizeof(INT_MEM->last_error_message),
				"Value of parameter %s is less than minimum.",
				(&static_model_info)->ParametersInfo[i].Name);
			static_model_info.LastErrorMessage = INT_MEM->last_error_message;
			return INT_MEM->last_error_message;
		} else if (pInstanceCapi->Parameters[i] > (&static_model_info)->ParametersInfo[i].MaxValue) {
			_snprintf(INT_MEM->last_error_message, sizeof(INT_MEM->last_error_message),
				"Value of parameter %s is greater than maximum.",
				(&static_model_info)->ParametersInfo[i].Name);
			static_model_info.LastErrorMessage = INT_MEM->last_error_message;
			return INT_MEM->last_error_message;
		}
	}

	return NULL;
}


/* Model_Loadflow */
extern DLL_EXPORT const char* __cdecl Model_Loadflow(InstanceExtSimEnvCapi *pInstanceCapi)
{
	// not used, nothing implemented
	return NULL;
}


/* Model_Initialize */
extern DLL_EXPORT const char* __cdecl Model_Initialize(InstanceExtSimEnvCapi *pInstanceCapi)
{

	double in_signal;
	in_signal = pInstanceCapi->ExternalInputs[0];	
	INT_MEM->OutputOrder = pInstanceCapi->Parameters[0];
	INT_MEM->OutputOrderMagnitude=0.;
	INT_MEM->counter = 0;

	// Do some minimal parameter sanity checks; further checks should be implemented throughout the control code;
	// model numerical stability is not ensured if the fallback value is chosen, so no convergence may appear depending on various parameters (e.g. sample rate)
	// Nevertheless, division by zero is avoided so no crashes are expected. 
	if (INT_MEM->OutputOrder <= 0) INT_MEM->OutputOrder = 0;

	INT_MEM->debug = 0;
	double debug1 = INT_MEM->debug;
	// Initialise outputs
	pInstanceCapi->ExternalOutputs[0] = 0.;
	pInstanceCapi->ExternalOutputs[1] = debug1;

	return NULL;
}


/* Model_Outputs */
extern DLL_EXPORT const char* __cdecl Model_Outputs(InstanceExtSimEnvCapi *pInstanceCapi, uint32_T IsMajorTimeStep)
{


	// Assign outputs
	pInstanceCapi->ExternalOutputs[0] = INT_MEM->OutputOrderMagnitude;
	pInstanceCapi->ExternalOutputs[1] = INT_MEM->debug;
	return NULL;
}


/* Model_Update */
extern DLL_EXPORT const char* __cdecl Model_Update(InstanceExtSimEnvCapi *pInstanceCapi)
{	
	int BUF_SIZ = 256; // hardcoded buffer size
	double in_signal;

	// Get input signal
	in_signal = pInstanceCapi->ExternalInputs[0];	

	int counter = INT_MEM->counter;	
	INT_MEM->data1[counter] = in_signal; // remove averaging
	INT_MEM->debug = INT_MEM->data1[counter];
	INT_MEM->counter++;
	if (INT_MEM->counter >= BUF_SIZ)
	{
		int harmonicOrder = INT_MEM->OutputOrder;
		INT_MEM->counter = 0;
		double OutputOrderMagnitude;
		OutputOrderMagnitude = 0.;
		double twopi = 6.28318530718;
		int iLength = BUF_SIZ;
		double dLength = BUF_SIZ;
		double argBase = harmonicOrder*twopi / dLength;
		double re = 0.;
		double im = 0.;
		for (int i=0; i < iLength; i++) {
			double phi = i * argBase;
			re += INT_MEM->data1[i] * cos(phi);
			im -= INT_MEM->data1[i] * sin(phi);
		}
		re = 2.0 * re / dLength;
		im = 2.0 * im / dLength;
		double mag = sqrt(re*re + im*im);
		INT_MEM->OutputOrderMagnitude = mag;
		
	
	}

	return NULL;
}



/* Model_Derivatives, not used current version, return NULL */
extern DLL_EXPORT const char* __cdecl Model_Derivatives(InstanceExtSimEnvCapi *pInstanceCapi)
{	
	return NULL;
}


/* Model_Terminate */
extern DLL_EXPORT const char* __cdecl Model_Terminate(InstanceExtSimEnvCapi *pInstanceCapi)
{	
	/* free ExternalInputs */
	free(pInstanceCapi->ExternalInputs);
	pInstanceCapi->ExternalInputs = NULL;

	/* free ExternalOutputs */
	free(pInstanceCapi->ExternalOutputs);
	pInstanceCapi->ExternalOutputs = NULL;

	/* free Parameters */
	free(pInstanceCapi->Parameters);
	pInstanceCapi->Parameters = NULL;

	/* free ContinuousStates */
	free(pInstanceCapi->ContinuousStates);
	pInstanceCapi->ContinuousStates = NULL;

	/* free StateDerivatives */
	free(pInstanceCapi->StateDerivatives);
	pInstanceCapi->StateDerivatives = NULL;

	/* free MiscStates */
	free(pInstanceCapi->MiscStates);
	pInstanceCapi->MiscStates = NULL;

	/* free model instance */
	free(pInstanceCapi);
	pInstanceCapi = NULL;

	return NULL;
}
